from plumber import (
    Behavior,
    default,
    plumb,
)
from pyramid.i18n import TranslationStringFactory
from ..model.utils import ugm_general

_ = TranslationStringFactory('cone.ugm')


class AutoIncrementForm(Behavior):
    """Plumbing behavior for setting user id by auto increment logic.

    For user add form.
    """

    @default
    @property
    def autoincrement_support(self):
        cfg = ugm_general(self.model)
        return cfg.attrs['user_id_autoincrement'] == 'True'

    @default
    @property
    def next_principal_id(self):
        cfg = ugm_general(self.model)
        prefix = cfg.attrs['user_id_autoincrement_prefix']
        default = int(cfg.attrs['user_id_autoincrement_start'])
        search = u'%s*' % prefix
        backend = self.model.parent.backend
        backend.invalidate()
        result = backend.search(attrlist=['id'], criteria={'id': search})
        principlal_ids = [_[1]['id'][0] for _ in result]
        matching = list()
        for principal_id in principlal_ids:
            if prefix:
                principal_id = principal_id[len(prefix):]
            try:
                principal_id = int(principal_id)
            except ValueError:
                continue
            matching.append(principal_id)
        if not matching:
            principal_id = default
        else:
            principal_id = sorted(matching)[-1] + 1
        if principal_id < default:
            principal_id = default
        return u'%s%i' % (prefix, principal_id)

    @plumb
    def prepare(_next, self):
        """Hook after prepare and set 'id' disabled.
        """
        _next(self)
        if not self.autoincrement_support:
            return
        id_field = self.form['id']
        del id_field.attrs['required']
        id_field.attrs['disabled'] = 'disabled'
        id_field.getter = _('auto_incremented', 'auto incremented')

    @plumb
    def save(_next, self, widget, data):
        if self.autoincrement_support:
            data['id'].extracted = self.next_principal_id
        _next(self, widget, data)



import time
from datetime import datetime
from plumber import (
    Behavior,
    plumb,
)
from pyramid.security import has_permission
from pyramid.i18n import TranslationStringFactory
from yafowil.base import (
    factory,
    UNSET,
    fetch_value,
)
from yafowil.common import (
    generic_extractor,
    generic_required_extractor,
)
from yafowil.utils import cssid
from yafowil.widget.datetime.widget import (
    format_date,
    format_time,
    render_datetime_input,
    datetime_display_renderer,
    datetime_extractor,
)
from ..model.utils import ugm_general

_ = TranslationStringFactory('cone.ugm')


def expiration_extractor(widget, data):
    """Extract expiration information.

    - If active flag not set, Account is disabled (value 0).
    - If active flag set and value is UNSET, account never expires.
    - If active flag set and datetime choosen, account expires at given
      datetime.
    - Timestamp in seconds since epoch is returned.
    """
    active = int(data.request.get('%s.active' % widget.name, '0'))
    if not active:
        return 0
    expires = data.extracted
    if expires:
        return time.mktime(expires.utctimetuple())
    return UNSET


def expiration_edit_renderer(widget, data):
    tag = data.tag
    active_attrs = dict()
    active_attrs['id'] = cssid(widget, 'checkbox')
    active_attrs['type'] = 'checkbox'
    active_attrs['name'] = '%s.active' % widget.name
    active_attrs['value'] = '1'
    value = fetch_value(widget, data)
    if value == 8639913600:
        value = UNSET
    if value != 0:
        active_attrs['checked'] = 'checked'
    active = tag('input', **active_attrs)
    until = tag('label', u'until')
    locale = widget.attrs['locale']
    if callable(locale):
        locale = locale(widget, data)
    date = None
    time = widget.attrs['time']
    if value in [0, UNSET]:
        date = ''
    else:
        date = datetime.fromtimestamp(value)
        if time:
            time = format_time(date)
        date = format_date(date, locale, widget.attrs['delimiter'])
    expires = render_datetime_input(widget, data, date, time)
    return tag('div', active + until + expires, class_='expiration-widget')


def expiration_display_renderer(widget, data):
    tag = data.tag
    active_attrs = dict()
    active_attrs['id'] = cssid(widget, 'checkbox')
    active_attrs['type'] = 'checkbox'
    active_attrs['disabled'] = 'disabled'
    value = data.value
    if value != 0:
        active_attrs['checked'] = 'checked'
    active = tag('input', **active_attrs)
    until = tag('label', u'until')
    if value not in [0, UNSET]:
        value = datetime.fromtimestamp(value)
    expires = datetime_display_renderer(widget, data, value)
    if expires:
        expires = until + expires
    return tag('div', active + expires, class_='expiration-widget')


factory.register(
    'expiration',
    extractors=[generic_extractor, generic_required_extractor,
                datetime_extractor, expiration_extractor],
    edit_renderers=[expiration_edit_renderer],
    display_renderers=[expiration_display_renderer])


factory.doc['blueprint']['expiration'] = \
"""Add-on blueprint UGM expiration widget. Utilizes yafowil.widget.datetime.
"""

factory.defaults['expiration.class'] = 'expiration'

factory.defaults['expiration.datepicker_class'] = 'datepicker'

factory.defaults['expiration.datepicker'] = True

factory.defaults['expiration.time'] = False

factory.defaults['expiration.tzinfo'] = None

factory.defaults['expiration.delimiter'] = '.'

factory.defaults['expiration.locale'] = 'de'

factory.defaults['expiration.format'] = '%Y.%m.%d'
factory.doc['props']['expiration.format'] = \
"""Pattern accepted by ``datetime.strftime``.
"""


class ExpirationForm(Behavior):
    """Expiration field plumbing behavior for user forms.
    """

    @plumb
    def prepare(_next, self):
        """Hook after prepare and set expiration widget to
        ``self.form``.
        """
        _next(self)
        cfg = ugm_general(self.model)
        if cfg.attrs['users_account_expiration'] != 'True':
            return
        mode = 'edit'
        if not has_permission('manage_expiration',
                              self.model.parent,
                              self.request):
            mode = 'display'
        if self.action_resource == 'edit':
            attr = cfg.attrs['users_expires_attr']
            unit = int(cfg.attrs['users_expires_unit'])
            value = int(self.model.attrs.get(attr, 0))
            # if format days, convert to seconds
            if unit == 0:
                value *= 86400
        else:
            value = UNSET
        expires_widget = factory(
            'field:label:expiration',
            name='active',
            value=value,
            props={
                'label': _('active', 'Active'),
            },
            mode=mode
        )
        save_widget = self.form['save']
        self.form.insertbefore(expires_widget, save_widget)

    @plumb
    def save(_next, self, widget, data):
        if has_permission('manage_expiration',
                          self.model.parent,
                          self.request):
            cfg = ugm_general(self.model)
            if cfg.attrs['users_account_expiration'] == 'True':
                attr = cfg.attrs['users_expires_attr']
                unit = int(cfg.attrs['users_expires_unit'])
                value = data.fetch('userform.active').extracted
                if value is UNSET:
                    if unit == 0:
                        value = 99999
                    else:
                        value = 8639913600
                elif value != 0:
                    if unit == 0:
                        add = 0
                        if value % 86400 != 0:
                            add = 1
                        value /= 86400
                        value += add
                    value = int(value)
                self.model.attrs[attr] = str(value)
        _next(self, widget, data)



import logging
from plumber import plumber
from node.locking import locktree
from node.utils import instance_property
from pyramid.i18n import TranslationStringFactory
from cone.app.model import (
    BaseNode,
    Metadata,
    NodeInfo,
    registerNodeInfo,
    Properties,
)
from .localmanager import LocalManagerGroupsACL
from .group import Group
from .utils import ugm_backend
from ..browser.utils import unquote_slash

logger = logging.getLogger('cone.ugm')
_ = TranslationStringFactory('cone.ugm')


def groups_factory():
    return Groups()


class Groups(BaseNode):
    __metaclass__ = plumber
    __plumbing__ = LocalManagerGroupsACL

    node_info_name = 'groups'

    @instance_property
    def properties(self):
        props = Properties()
        props.in_navtree = True
        return props

    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _('groups_node', 'Groups')
        metadata.description = _('groups_node_description',
                                 'Container for Groups')
        return metadata

    @property
    def backend(self):
        return ugm_backend(self).groups

    @locktree
    def invalidate(self, key=None):
        if key is None:
            del self.backend.parent.storage['groups']
            self.clear()
            return
        self.backend.invalidate(key)
        try:
            del self[key]
        except KeyError:
            pass

    @locktree
    def __call__(self):
        self.backend()

    @locktree
    def __iter__(self):
        try:
            return self.backend.__iter__()
        except Exception, e:
            logger.error(str(e))
            return iter(list())

    iterkeys = __iter__

    @locktree
    def __getitem__(self, name):
        # XXX: temporary hack until paster/webob/pyramid handle urllib
        # quoted slashes in path components
        name = unquote_slash(name)
        try:
            return BaseNode.__getitem__(self, name)
        except KeyError:
            try:
                model = self.backend[name]
            except AttributeError:
                raise KeyError(name)
            group = Group(model, name, self)
            self[name] = group
            return group


info = NodeInfo()
info.title = _('groups_node', 'Groups')
info.description = _('groups_node_description',
                     'Container for Groups')
info.node = Groups
info.addables = ['group']
info.icon = 'cone.ugm.static/images/groups16_16.png'
registerNodeInfo('groups', info)



import os
import doctest
import interlude
import pprint
import pkg_resources
import unittest2 as unittest
from plone.testing import layered
from cone.app.testing import Security
from node.ext.ldap.testing import LDIF_groupOfNames_10_10


class UGMLayer(Security):
    defaultBases = (LDIF_groupOfNames_10_10, )

    def setUp(self, args=None):
        super(UGMLayer, self).setUp(args)
        import cone.ugm
        path = pkg_resources.resource_filename('cone.ugm.tests', 'ldap.xml')
        cone.ugm.model.utils.LDAP_CFG_FILE = path
        cone.ugm.model.settings._invalidate_ugm_settings(cone.app.get_root())
        ugm = cone.ugm.model.utils.ugm_backend(cone.app.get_root())
        roles = ['viewer', 'editor', 'admin', 'manager']

        def create_user(uid):
            data = {
                'cn': uid,
                'sn': uid,
                'mail': '%s@example.com' % uid,
            }
            user = ugm.users.create(uid, **data)
            ugm()
            ugm.users.passwd(uid, None, 'secret')
            return user
        for uid in ['viewer', 'editor', 'admin', 'manager', 'max', 'sepp']:
            user = create_user(uid)
            if uid in roles:
                user.add_role(uid)
        for uid in ['localmanager_1', 'localmanager_2']:
            create_user(uid)
        for gid, uid in [('admin_group_1', 'localmanager_1'),
                         ('admin_group_2', 'localmanager_2')]:
            group = ugm.groups.create(gid)
            group.add(uid)
        ugm()

    def tearDown(self):
        super(UGMLayer, self).tearDown()
        import cone.app
        ugm = cone.app.cfg.auth
        for uid in ['viewer', 'editor', 'admin', 'manager', 'max', 'sepp',
                    'localmanager_1', 'localmanager_2']:
            del ugm.users[uid]
        for gid in ['admin_group_1', 'admin_group_2']:
            del ugm.groups[gid]
        ugm.users()

    def make_app(self):
        base_path = os.path.split(__file__)[0]
        ldap_config = os.path.join(base_path, 'ldap.xml')
        localmanager_config = os.path.join(base_path, 'localmanager.xml')
        super(UGMLayer, self).make_app(**{
            'cone.auth_impl': 'node.ext.ldap',
            'cone.plugins': 'node.ext.ugm\ncone.ugm',
            'cone.ugm.ldap_config': ldap_config,
            'cone.ugm.localmanager_config': localmanager_config,
        })
        LDIF_groupOfNames_10_10.gcfg.attrmap['cn'] = 'cn'


ugm_layer = UGMLayer()


DOCFILES = [
    ('../model/settings.rst', ugm_layer),
    ('../model/utils.rst', ugm_layer),
    ('../model/users.rst', ugm_layer),
    ('../model/groups.rst', ugm_layer),
    ('../model/user.rst', ugm_layer),
    ('../model/group.rst', ugm_layer),
    ('../model/localmanager.rst', ugm_layer),
    ('../browser/__init__.rst', ugm_layer),
    ('../browser/utils.rst', ugm_layer),
    ('../browser/users.rst', ugm_layer),
    ('../browser/groups.rst', ugm_layer),
    ('../browser/root.rst', ugm_layer),
    ('../browser/user.rst', ugm_layer),
    ('../browser/group.rst', ugm_layer),
    ('../browser/actions.rst', ugm_layer),
    ('../browser/settings.rst', ugm_layer),
    ('../browser/remote.rst', ugm_layer),
    ('../browser/expires.rst', ugm_layer),
    ('../browser/roles.rst', ugm_layer),
    ('../browser/autoincrement.rst', ugm_layer),
    ('../browser/portrait.rst', ugm_layer),
]


optionflags = doctest.NORMALIZE_WHITESPACE | \
              doctest.ELLIPSIS | \
              doctest.REPORT_ONLY_FIRST_FAILURE


print """
*******************************************************************************
If testing while development fails, please check if memcached is installed and
stop it if running.
*******************************************************************************
"""


def test_suite():
    suite = unittest.TestSuite()
    suite.addTests([
        layered(
            doctest.DocFileSuite(
                docfile,
                globs={
                    'interact': interlude.interact,
                    'pprint': pprint.pprint,
                    'pp': pprint.pprint,
                    },
                optionflags=optionflags,
                ),
            layer=layer,
            )
        for docfile, layer in DOCFILES
        ])
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')                 #pragma NO COVERAGE



from cone.tile import (
    Tile,
    registerTile,
    render_tile,
)
from cone.app.browser.layout import ProtectedContentTile


registerTile('content',
             'cone.ugm:browser/templates/columns.pt',
             class_=ProtectedContentTile,
             permission='login',
             strict=False)


class Column(Tile):
    """Abstract column.
    """

    def _render(self, model, name):
        return render_tile(model, self.request, name)



from plumber import plumber
from pyramid.security import has_permission
from pyramid.i18n import TranslationStringFactory
from yafowil.base import ExtractionError
from yafowil.utils import UNSET
from cone.tile import (
    tile,
    Tile,
)
from cone.app.browser.utils import (
    make_url,
    make_query,
)
from cone.app.browser.form import Form
from cone.app.browser.authoring import (
    AddBehavior,
    EditBehavior,
)
from cone.app.browser.ajax import AjaxAction
from ..model.group import Group
from ..model.utils import (
    ugm_general,
    ugm_groups,
)
from . import form_field_definitions
from .columns import Column
from .listing import ColumnListing
from .roles import PrincipalRolesForm
from .authoring import (
    AddFormFiddle,
    EditFormFiddle,
)
from .principal import PrincipalForm
from webob.exc import HTTPFound

_ = TranslationStringFactory('cone.ugm')


@tile('leftcolumn', interface=Group, permission='view')
class GroupLeftColumn(Column):

    add_label = _('add_group', 'Add Group')

    @property
    def can_add(self):
        return has_permission('add_group', self.model.parent, self.request)

    def render(self):
        setattr(self.request, '_curr_listing_id', self.model.name)
        return self._render(self.model.parent, 'leftcolumn')


@tile('rightcolumn', 'templates/right_column.pt',
      interface=Group, permission='view')
class GroupRightColumn(Tile):

    @property
    def default_widget(self):
        settings = ugm_general(self.model)
        return settings.attrs['default_membership_assignment_widget']


class Principals(object):
    """Descriptor to return principal items for listing
    """
    def __init__(self,
                 members_only=False,
                 available_only=False):
        self.members_only = members_only
        self.available_only = available_only

    def __get__(self, obj, objtype=None):
        if obj is None:
            return self

        appgroup = obj.model
        group = appgroup.model
        member_ids = group.keys()

        # always True if we list members only, otherwise will be set
        # in the loop below
        related = self.members_only

        available_only = self.available_only

        if related and available_only:
            raise Exception(u"Invalid object settings.")

        # XXX: so far only users as members of groups, for
        # group-in-group we need to prefix groups
        if related:
            users = group.users
        else:
            # XXX: LDAP query here.
            users = group.root.users.values()
            if available_only:
                users = [u for u in users if not u.name in member_ids]

        # reduce for local manager
        if obj.model.local_manager_consider_for_user:
            local_uids = obj.model.local_manager_target_uids
            users = [u for u in users if u.name in local_uids]

        attrlist = obj.user_attrs
        sort_attr = obj.user_default_sort_column

        ret = list()

        can_change = has_permission(
            'manage_membership', obj.model.parent, obj.request)

        for user in users:
            uid = user.name
            attrs = user.attrs

            item_target = make_url(obj.request, path=user.path[1:])
            action_query = make_query(id=uid)
            action_target = make_url(obj.request,
                                     node=appgroup,
                                     query=action_query)

            if not self.members_only:
                related = uid in member_ids

            actions = list()
            if can_change:
                action_id = 'add_item'
                action_enabled = not bool(related)
                action_title = _('add_user_to_selected_group',
                                 'Add user to selected group')
                add_item_action = obj.create_action(
                    action_id, action_enabled, action_title, action_target)
                actions.append(add_item_action)

                action_id = 'remove_item'
                action_enabled = bool(related)
                action_title = _('remove_user_from_selected_group',
                                 'Remove user from selected group')
                remove_item_action = obj.create_action(
                    action_id, action_enabled, action_title, action_target)
                actions.append(remove_item_action)

            vals = [obj.extract_raw(attrs, attr) for attr in attrlist]
            sort = obj.extract_raw(attrs, sort_attr)
            content = obj.item_content(*vals)
            current = False
            item = obj.create_item(sort, item_target, content,
                                   current, actions)
            ret.append(item)
        return ret


@tile('columnlisting', 'templates/column_listing.pt',
      interface=Group, permission='view')
class UsersOfGroupColumnListing(ColumnListing):
    css = 'users'
    slot = 'rightlisting'
    list_columns = ColumnListing.user_list_columns
    # XXX: Why items and not keys() / __iter__?
    # used to be a readonly property
    query_items = Principals(members_only=True)
    batchname = 'rightbatch'


@tile('allcolumnlisting', 'templates/column_listing.pt',
      interface=Group, permission='view')
class AllUsersColumnListing(ColumnListing):
    css = 'users'
    slot = 'rightlisting'
    list_columns = ColumnListing.user_list_columns
    # XXX: Why items and not keys() / __iter__?
    # used to be a readonly property
    query_items = Principals()
    batchname = 'rightbatch'

    @property
    def ajax_action(self):
        return 'allcolumnlisting'


@tile('inoutlisting', 'templates/in_out.pt',
      interface=Group, permission='view')
class InOutListing(ColumnListing):
    selected_items = Principals(members_only=True)
    available_items = Principals(available_only=True)

    @property
    def user_attrs(self):
        settings = ugm_general(self.model)
        return [settings.attrs['user_display_name_attr']]

    @property
    def user_default_sort_column(self):
        settings = ugm_general(self.model)
        return settings.attrs['user_display_name_attr']

    @property
    def display_control_buttons(self):
        return True
        #settings = ugm_general(self.model)
        #return settings.attrs['controls_membership_assignment_widget']


class GroupForm(PrincipalForm):
    form_name = 'groupform'

    @property
    def form_attrmap(self):
        settings = ugm_groups(self.model)
        return settings.attrs.groups_form_attrmap

    @property
    def form_field_definitions(self):
        return form_field_definitions.group

    def exists(self, widget, data):
        group_id = data.extracted
        if group_id is UNSET:
            return data.extracted
        if group_id in self.model.parent.backend:
            message = _('group_already_exists',
                        default="Group ${gid} already exists.",
                        mapping={'gid': group_id})
            raise ExtractionError(message)
        return data.extracted


@tile('addform', interface=Group, permission="add_group")
class GroupAddForm(GroupForm, Form):
    __metaclass__ = plumber
    __plumbing__ = AddBehavior, PrincipalRolesForm, AddFormFiddle

    show_heading = False
    show_contextmenu = False

    def save(self, widget, data):
        settings = ugm_groups(self.model)
        attrmap = settings.attrs.groups_form_attrmap
        extracted = dict()
        for key, val in attrmap.items():
            val = data.fetch('groupform.%s' % key).extracted
            if not val:
                continue
            extracted[key] = val
        groups = self.model.parent.backend
        gid = extracted.pop('id')
        #group = groups.create(gid, **extracted)
        groups.create(gid, **extracted)
        self.request.environ['next_resource'] = gid
        groups()
        self.model.parent.invalidate()
        # XXX: access already added user after invalidation.
        #      if not done, there's some kind of race condition with ajax
        #      continuation. figure out why.
        self.model.parent[gid]

    def next(self, request):
        next_resource = self.request.environ.get('next_resource')
        if next_resource:
            url = make_url(request.request,
                           node=self.model,
                           resource=next_resource)
        else:
            url = make_url(request.request, node=self.model)
        if self.ajax_request:
            return [
                AjaxAction(url, 'leftcolumn', 'replace', '.left_column'),
                AjaxAction(url, 'rightcolumn', 'replace', '.right_column'),
            ]
        return HTTPFound(location=url)


@tile('editform', interface=Group, permission="edit_group", strict=False)
class GroupEditForm(GroupForm, Form):
    __metaclass__ = plumber
    __plumbing__ = EditBehavior, PrincipalRolesForm, EditFormFiddle

    show_heading = False
    show_contextmenu = False

    def save(self, widget, data):
        settings = ugm_groups(self.model)
        attrmap = settings.attrs.groups_form_attrmap
        for key in attrmap:
            if key in ['id']:
                continue
            extracted = data.fetch('groupform.%s' % key).extracted
            if not extracted:
                if key in self.model.attrs:
                    del self.model.attrs[key]
            else:
                self.model.attrs[key] = extracted
        self.model.model.context()

    def next(self, request):
        url = make_url(request.request, node=self.model)
        if self.ajax_request:
            return [
                AjaxAction(url, 'leftcolumn', 'replace', '.left_column'),
                AjaxAction(url, 'rightcolumn', 'replace', '.right_column'),
            ]
        return HTTPFound(location=url)



from plumber import plumber
from odict import odict
from ldap.functions import explode_dn
from pyramid.view import view_config
from pyramid.i18n import (
    TranslationStringFactory,
    get_localizer,
)
from node.ext.ldap import (
    BASE,
    ONELEVEL,
    SUBTREE,
    LDAPNode,
)
from yafowil.base import (
    UNSET,
    ExtractionError,
)
from cone.tile import (
    tile,
    Tile,
    registerTile,
)
from cone.app.browser.layout import ProtectedContentTile
from cone.app.browser.form import (
    Form,
    YAMLForm,
)
from cone.app.browser.settings import SettingsBehavior
from cone.app.browser.ajax import (
    ajax_continue,
    ajax_message,
    AjaxAction,
)
from cone.app.browser.utils import (
    make_url,
    make_query,
)
from ..model.settings import (
    GeneralSettings,
    ServerSettings,
    UsersSettings,
    GroupsSettings,
    RolesSettings,
    LocalManagerSettings,
)

_ = TranslationStringFactory('cone.ugm')


class VocabMixin(object):

    scope_vocab = [
        (str(BASE), 'BASE'),
        (str(ONELEVEL), 'ONELEVEL'),
        (str(SUBTREE), 'SUBTREE'),
    ]


def encode_dn(dn):
    return dn.replace('=', '%%')


def decode_dn(dn):
    return dn.replace('%%', '=')


class CreateContainerTrigger(Tile):

    @property
    def creation_dn(self):
        raise NotImplementedError(u"Abstract ``CreateContainerTrigger`` "
                                  u"does not implement ``creation_dn``")

    @property
    def creation_target(self):
        dn = encode_dn(self.creation_dn)
        query = make_query(dn=dn)
        return make_url(self.request, node=self.model, query=query)

    @property
    def ldap_connectivity(self):
        return self.model.parent['ugm_server'].ldap_connectivity


class CreateContainerAction(Tile):

    @property
    def continuation(self):
        raise NotImplementedError(u"Abstract ``CreateContainerAction`` "
                                  u"does not implement ``continuation``")

    def render(self):
        try:
            message = self.create_container()
            ajax_message(self.request, message, 'info')
            continuation = self.continuation
            ajax_continue(self.request, continuation)
        except Exception, e:
            localizer = get_localizer(self.request)
            message = localizer.translate(
                _('cannot_create_container',
                  default="Can't create container: ${error}",
                  mapping={'error': str(e)}))
            ajax_message(self.request, message, 'error')
        return u''

    def create_container(self):
        dn = decode_dn(self.request.params.get('dn', ''))
        localizer = get_localizer(self.request)
        if not dn:
            message = localizer.translate(
                _('no_container_dn_defined', 'No container DN defined.'))
            raise Exception(message)
        if not dn.startswith('ou='):
            message = localizer.translate(
                _('expected_ou_as_rdn', "Expected 'ou' as RDN Attribute."))
            raise Exception(message)
        props = self.model.parent['ugm_server'].ldap_props
        try:
            parent_dn = ','.join(explode_dn(dn)[1:])
        except Exception:
            message = localizer.translate(_('invalid_dn', 'Invalid DN.'))
            raise Exception(message)
        rdn = explode_dn(dn)[0]
        node = LDAPNode(parent_dn, props)
        if node is None:
            message = localizer.translate(
                _('parent_not_found', "Parent not found. Can't continue."))
            raise Exception(message)
        node[rdn] = LDAPNode()
        node[rdn].attrs['objectClass'] = ['organizationalUnit']
        node()
        self.model.invalidate()
        message = localizer.translate(
                _('created_principal_container',
                  default="Created ${rdn}",
                  mapping={'rdn': rdn}))
        return message


registerTile('content',
             'cone.ugm:browser/templates/general_settings.pt',
             class_=ProtectedContentTile,
             interface=GeneralSettings,
             permission='manage')


@tile('editform', interface=GeneralSettings, permission="manage")
class GeneralSettingsForm(Form):
    __metaclass__ = plumber
    __plumbing__ = SettingsBehavior, YAMLForm

    action_resource = u'edit'
    form_template = 'cone.ugm.browser:forms/general_settings.yaml'

    @property
    def message_factory(self):
        return _

    def save(self, widget, data):
        model = self.model
        for attr_name in ['default_membership_assignment_widget',
                          'user_display_name_attr',
                          'group_display_name_attr',
                          'users_account_expiration',
                          'users_expires_attr',
                          'users_expires_unit',
                          'user_id_autoincrement',
                          'user_id_autoincrement_prefix',
                          'user_id_autoincrement_start',
                          'users_portrait',
                          'users_portrait_attr',
                          'users_portrait_accept',
                          'users_portrait_width',
                          'users_portrait_height',
                          'users_local_management_enabled']:
            val = data.fetch('ugm_general.%s' % attr_name).extracted
            setattr(model.attrs, attr_name, val)
        model()
        model.invalidate()


@tile('content', 'templates/server_settings.pt',
      interface=ServerSettings, permission='manage')
class ServerSettingsTile(ProtectedContentTile):

    @property
    def ldap_status(self):
        if self.model.ldap_connectivity:
            return 'OK'
        return _('server_down', 'Down')


@tile('editform', interface=ServerSettings, permission="manage")
class ServerSettingsForm(Form):
    __metaclass__ = plumber
    __plumbing__ = SettingsBehavior, YAMLForm

    action_resource = u'edit'
    form_template = 'cone.ugm.browser:forms/server_settings.yaml'

    @property
    def message_factory(self):
        return _

    def save(self, widget, data):
        model = self.model
        for attr_name in ['uri', 'user']:
            val = data.fetch('ldap_server_settings.%s' % attr_name).extracted
            setattr(model.attrs, attr_name, val)
        cache = data.fetch('ldap_server_settings.cache').extracted
        cache = str(int(cache))
        setattr(model.attrs, 'cache', cache)
        password = data.fetch('ldap_server_settings.password').extracted
        if password is not UNSET:
            setattr(model.attrs, 'password', password)
        model()
        model.invalidate()


@tile('content', 'templates/users_settings.pt',
      interface=UsersSettings, permission='manage')
class UsersSettingsTile(ProtectedContentTile, CreateContainerTrigger):

    @property
    def creation_dn(self):
        return self.model.attrs.users_dn

    @property
    def ldap_users(self):
        if self.model.ldap_users_container_valid:
            return 'OK'
        return _('inexistent', 'Inexistent')


@tile('create_container', interface=UsersSettings, permission='manage')
class UsersCreateContainerAction(CreateContainerAction):

    @property
    def continuation(self):
        url = make_url(self.request, node=self.model)
        return AjaxAction(url, 'content', 'inner', '.ugm_users')


@tile('editform', interface=UsersSettings, permission="manage")
class UsersSettingsForm(Form, VocabMixin):
    __metaclass__ = plumber
    __plumbing__ = SettingsBehavior, YAMLForm

    action_resource = u'edit'
    form_template = 'cone.ugm.browser:forms/users_settings.yaml'

    @property
    def message_factory(self):
        return _

    @property
    def users_aliases_attrmap(self):
        attrs = self.model.attrs
        users_aliases_attrmap = odict()
        users_aliases_attrmap['rdn'] = attrs.users_aliases_attrmap.get('rdn')
        users_aliases_attrmap['id'] = attrs.users_aliases_attrmap.get('id')
        users_aliases_attrmap['login'] = \
            attrs.users_aliases_attrmap.get('login')
        return users_aliases_attrmap

    def save(self, widget, data):
        model = self.model
        for attr_name in ['users_dn',
                          'users_scope',
                          'users_query',
                          'users_object_classes',
                          'users_aliases_attrmap',
                          'users_form_attrmap',
                          'users_listing_columns',
                          'users_listing_default_column',
                          'users_exposed_attributes']:
            val = data.fetch('ldap_users_settings.%s' % attr_name).extracted
            if attr_name == 'users_object_classes':
                val = [v.strip() for v in val.split(',') if v.strip()]
            setattr(model.attrs, attr_name, val)
        model()
        model.invalidate()


@tile('content', 'templates/groups_settings.pt',
      interface=GroupsSettings, permission='manage')
class GroupsSettingsTile(ProtectedContentTile, CreateContainerTrigger):

    @property
    def creation_dn(self):
        return self.model.attrs.groups_dn

    @property
    def ldap_groups(self):
        if self.model.ldap_groups_container_valid:
            return 'OK'
        return _('inexistent', 'Inexistent')


@tile('create_container', interface=GroupsSettings, permission='manage')
class GroupsCreateContainerAction(CreateContainerAction):

    @property
    def continuation(self):
        url = make_url(self.request, node=self.model)
        return AjaxAction(url, 'content', 'inner', '.ugm_groups')


@tile('editform', interface=GroupsSettings, permission="manage")
class GroupsSettingsForm(Form, VocabMixin):
    __metaclass__ = plumber
    __plumbing__ = SettingsBehavior, YAMLForm

    action_resource = u'edit'
    form_template = 'cone.ugm.browser:forms/groups_settings.yaml'

    @property
    def message_factory(self):
        return _

    @property
    def groups_aliases_attrmap(self):
        attrs = self.model.attrs
        groups_aliases_attrmap = odict()
        groups_aliases_attrmap['rdn'] = attrs.groups_aliases_attrmap.get('rdn')
        groups_aliases_attrmap['id'] = attrs.groups_aliases_attrmap.get('id')
        return groups_aliases_attrmap

    def save(self, widget, data):
        model = self.model
        for attr_name in ['groups_dn',
                          'groups_scope',
                          'groups_query',
                          'groups_object_classes',
                          'groups_aliases_attrmap',
                          'groups_form_attrmap',
                          #'groups_relation',
                          'groups_listing_columns',
                          'groups_listing_default_column']:
            val = data.fetch('ldap_groups_settings.%s' % attr_name).extracted
            if attr_name == 'groups_object_classes':
                val = [v.strip() for v in val.split(',') if v.strip()]
            setattr(model.attrs, attr_name, val)
        model()
        model.invalidate()


@tile('content', 'templates/roles_settings.pt',
      interface=RolesSettings, permission='manage')
class RolesSettingsTile(ProtectedContentTile, CreateContainerTrigger):

    @property
    def creation_dn(self):
        return self.model.attrs.roles_dn

    @property
    def ldap_roles(self):
        if self.model.ldap_roles_container_valid:
            return 'OK'
        return _('inexistent', 'Inexistent')


@tile('create_container', interface=RolesSettings, permission='manage')
class RolesCreateContainerAction(CreateContainerAction):

    @property
    def continuation(self):
        url = make_url(self.request, node=self.model)
        return AjaxAction(url, 'content', 'inner', '.ugm_roles')


@tile('editform', interface=RolesSettings, permission="manage")
class RolesSettingsForm(Form, VocabMixin):
    __metaclass__ = plumber
    __plumbing__ = SettingsBehavior, YAMLForm

    action_resource = u'edit'
    form_template = 'cone.ugm.browser:forms/roles_settings.yaml'

    @property
    def message_factory(self):
        return _

    @property
    def roles_aliases_attrmap(self):
        attrs = self.model.attrs
        roles_aliases_attrmap = odict()
        roles_aliases_attrmap['rdn'] = attrs.roles_aliases_attrmap.get('rdn')
        roles_aliases_attrmap['id'] = attrs.roles_aliases_attrmap.get('id')
        return roles_aliases_attrmap

    def save(self, widget, data):
        model = self.model
        for attr_name in ['roles_dn',
                          'roles_scope',
                          'roles_query',
                          'roles_object_classes',
                          'roles_aliases_attrmap',
                          'roles_form_attrmap',
                          #'roles_relation',
                          ]:
            val = data.fetch('ldap_roles_settings.%s' % attr_name).extracted
            if attr_name == 'roles_object_classes':
                val = [v.strip() for v in val.split(',') if v.strip()]
            setattr(model.attrs, attr_name, val)
        model()
        model.invalidate()


@tile('content', 'templates/localmanager_settings.pt',
      interface=LocalManagerSettings, permission='manage')
class LocalManagerSettingsTile(ProtectedContentTile):
    pass


@tile('editform', interface=LocalManagerSettings, permission="manage")
class LocalManagerSettingsForm(Form):
    __metaclass__ = plumber
    __plumbing__ = SettingsBehavior, YAMLForm

    action_resource = u'edit'
    form_template = 'cone.ugm.browser:forms/localmanager_settings.yaml'

    @property
    def message_factory(self):
        return _

    @property
    def rules_value(self):
        """Return value format:

            return [{
                'source': 'aaa',
                'targets': [{
                    'gid': 'bbb',
                    'default': False,
                }]
            }]
        """
        rules = list()
        items = self.model.attrs.items()
        items = sorted(items, key=lambda x: x[0])
        for source, defs in items:
            rule = dict()
            rule['source'] = source
            rule['targets'] = list()
            targets = sorted(defs['target'])
            for gid in targets:
                rule['targets'].append({
                    'gid': gid,
                    'default': gid in defs['default']
                })
            rules.append(rule)
        return rules

    def duplicate_rule(self, widget, data):
        """Check for duplicate rules.
        """
        source = data.extracted['source']
        if not source:
            return data.extracted
        exists = [source]
        for val in data.parent.values():
            if val.name == data.name:
                continue
            other = val.extracted['source']
            if other in exists:
                raise ExtractionError(_('localmanager_duplicate_rule_error'))
            exists.append(other)
        return data.extracted

    def target_not_source(self, widget, data):
        """Check whether source and target are same.
        """
        source = data.parent.parent.parent['source'].extracted
        if source == data.extracted:
            raise ExtractionError(_('localmanager_target_is_source_error'))
        return data.extracted

    def save(self, widget, data):
        """save rules.
        """
        attrs = self.model.attrs
        recent = attrs.keys()
        extracted = data.fetch('localmanager_settings.rules').extracted
        for entry in extracted:
            source = entry['source']
            if source in recent:
                recent.remove(source)
            targets = set()
            defaults = set()
            for target in entry['targets']:
                targets.add(target['gid'])
                if target['default']:
                    defaults.add(target['gid'])
            rule = {
                'target': list(targets),
                'default': list(defaults),
            }
            attrs[source] = rule
        for source in recent:
            del attrs[source]
        self.model()


@view_config(name='group_id_vocab', accept='application/json',
             renderer='json', permission='manage')
def group_id_vocab(model, request):
    term = request.params['term']
    if len(term) < 2:
        return []
    backend = model.root['groups'].backend
    return backend.search(criteria={'id': '%s*' % term})



from cone.app.model import Properties
from .users import Users
from .user import User
from .groups import Groups
from .group import Group


# user and group factory defaults
factory_defaults = Properties()
factory_defaults.user = dict()
factory_defaults.group = dict()
factory_defaults.role = dict()



import os
from lxml import etree
from plumber import (
    Behavior,
    plumber,
    plumb,
    default,
    finalize,
)
from node.behaviors import (
    Nodify,
    DictStorage,
)
from pyramid.security import (
    Allow,
    authenticated_userid,
)
from pyramid.threadlocal import get_current_request
from cone.app import security


class LocalManagerConfig(DictStorage):
    """Local Management configuration storage.
    """

    @finalize
    def load(self):
        path = self.file_path
        if not path or not os.path.exists(path):
            return
        with open(path, 'r') as handle:
            tree = etree.parse(handle)
        root = tree.getroot()
        for rule in root.getchildren():
            new_rule = self.storage[rule.tag] = dict()
            for prop in rule.getchildren():
                for tag_name in ['target', 'default']:
                    if prop.tag == tag_name:
                        new_rule[tag_name] = list()
                        for group in prop.getchildren():
                            new_rule[tag_name].append(group.text)

    @finalize
    def __call__(self):
        root = etree.Element('localmanager')
        for gid, rule in self.storage.items():
            group = etree.SubElement(root, gid)
            for tag_name in ['target', 'default']:
                elem = etree.SubElement(group, tag_name)
                for gid in rule[tag_name]:
                    item = etree.SubElement(elem, 'item')
                    item.text = gid
        with open(self.file_path, 'w') as handle:
            handle.write(etree.tostring(root, pretty_print=True))


class LocalManagerConfigAttributes(object):
    __metaclass__ = plumber
    __plumbing__ = (
        Nodify,
        LocalManagerConfig,
    )

    def __init__(self, path):
        self.file_path = path
        self.load()


class LocalManager(Behavior):
    """Behavior providing local manager information for authenticated user.
    """

    @finalize
    @property
    def local_management_enabled(self):
        """Flag whether local management is enabled.
        """
        general_settings = self.root['settings']['ugm_general']
        return general_settings.attrs.users_local_management_enabled == 'True'

    @finalize
    @property
    def local_manager_consider_for_user(self):
        """Flag whether local manager ACL should be considered for current
        authenticated user.
        """
        if not self.local_management_enabled:
            return False
        request = get_current_request()
        if authenticated_userid(request) == security.ADMIN_USER:
            return False
        roles = security.authenticated_user(request).roles
        if 'admin' in roles or 'manager' in roles:
            return False
        return True

    @finalize
    @property
    def local_manager_gid(self):
        """Group id of local manager group of current authenticated member.

        Currently a user can be assigned only to one local manager group. If
        mode than one local manager group is configured, an error is raised.
        """
        config = self.root['settings']['ugm_localmanager'].attrs
        user = security.authenticated_user(get_current_request())
        if not user:
            return None
        gids = user.group_ids
        adm_gids = list()
        for gid in gids:
            rule = config.get(gid)
            if rule:
                adm_gids.append(gid)
        if len(adm_gids) == 0:
            return None
        if len(adm_gids) > 1:
            msg = (u"Authenticated member defined in local manager "
                   u"groups %s but only one management group allowed for "
                   u"each user. Please contact System Administrator in "
                   u"order to fix this problem.")
            exc = msg % ', '.join(["'%s'" % gid for gid in adm_gids])
            raise Exception(exc)
        return adm_gids[0]

    @finalize
    @property
    def local_manager_rule(self):
        """Return rule for local manager.
        """
        adm_gid = self.local_manager_gid
        if not adm_gid:
            return None
        config = self.root['settings']['ugm_localmanager'].attrs
        return config[adm_gid]

    @finalize
    @property
    def local_manager_default_gids(self):
        """Return default group id's for local manager.
        """
        rule = self.local_manager_rule
        if not rule:
            return list()
        return rule['default']

    @finalize
    @property
    def local_manager_target_gids(self):
        """Target group id's for local manager.
        """
        rule = self.local_manager_rule
        if not rule:
            return list()
        return rule['target']

    @finalize
    @property
    def local_manager_target_uids(self):
        """Target uid's for local manager.
        """
        groups = self.root['groups'].backend
        managed_uids = set()
        for gid in self.local_manager_target_gids:
            group = groups.get(gid)
            if group:
                managed_uids.update(group.member_ids)
        return list(managed_uids)

    @finalize
    def local_manager_is_default(self, adm_gid, gid):
        """Check whether gid is default group for local manager group.
        """
        config = self.root['settings']['ugm_localmanager'].attrs
        rule = config[adm_gid]
        if not gid in rule['target']:
            raise Exception(u"group '%s' not managed by '%s'" % (gid, adm_gid))
        return gid in rule['default']


class LocalManagerACL(LocalManager):
    """Behavior providing ACL's by local manager configuration.
    """

    @default
    @property
    def local_manager_acl(self):
        raise NotImplementedError(u"Abstract ``LocalManagerACL`` does not "
                                  u"implement ``local_manager_acl``")

    @plumb
    @property
    def __acl__(_next, self):
        acl = _next(self)
        if not self.local_manager_consider_for_user:
            return acl
        return self.local_manager_acl + acl


class LocalManagerUsersACL(LocalManagerACL):

    @finalize
    @property
    def local_manager_acl(self):
        if not self.local_manager_target_gids:
            return []
        permissions = ['view', 'add', 'add_user', 'edit', 'edit_user',
                       'manage_expiration', 'manage_membership']
        return [(Allow,
                 authenticated_userid(get_current_request()),
                 permissions)]


class LocalManagerUserACL(LocalManagerACL):

    @finalize
    @property
    def local_manager_acl(self):
        # if self.name is None, User object was created by add model factory
        if self.name is not None:
            if not self.name in self.local_manager_target_uids:
                return []
        permissions = ['view', 'add', 'add_user', 'edit', 'edit_user',
                       'manage_expiration', 'manage_membership']
        return [(Allow,
                 authenticated_userid(get_current_request()),
                 permissions)]


class LocalManagerGroupsACL(LocalManagerACL):

    @finalize
    @property
    def local_manager_acl(self):
        if not self.local_manager_target_gids:
            return []
        permissions = ['view', 'manage_membership']
        return [(Allow,
                 authenticated_userid(get_current_request()),
                 permissions)]


class LocalManagerGroupACL(LocalManagerACL):

    @finalize
    @property
    def local_manager_acl(self):
        if not self.name in self.local_manager_target_gids:
            return []
        permissions = ['view', 'manage_membership']
        return [(Allow,
                 authenticated_userid(get_current_request()),
                 permissions)]



from pyramid.static import static_view
from yafowil.common import ascii_extractor
from cone.app.model import Properties


static_resources = static_view('static', use_subpath=True)


# user and group form field definitions for yafowil
# overwrite or extend in integration __init__
# XXX: future -> yafowil form field properties editor
# XXX: far future -> yafowil JS form editor
# XXX: user and group form schema definitions should then be resolved in order
#      yafowil browser based cfg -> default cfg -> general default
form_field_definitions = Properties()
form_field_definitions.user = dict()
form_field_definitions.group = dict()
_user = form_field_definitions.user
_group = form_field_definitions.group


_user['default'] = dict()
_user['default']['chain'] = 'field:label:error:text'
_user['default']['required'] = False
_user['default']['protected'] = False


_user['id'] = dict()
_user['id']['chain'] = 'field:*ascii:*exists:label:error:text'
_user['id']['props'] = dict()
_user['id']['props']['ascii'] = True
_user['id']['custom'] = dict()
_user['id']['custom']['ascii'] = ([ascii_extractor], [], [], [], [])
_user['id']['custom']['exists'] = (['context.exists'], [], [], [], [])
_user['id']['required'] = True
_user['id']['protected'] = True


_user['mail'] = dict()
_user['mail']['chain'] = 'field:label:error:email'
_user['mail']['required'] = False
_user['mail']['protected'] = False


_user['userPassword'] = dict()
_user['userPassword']['chain'] = 'field:label:error:password'
_user['userPassword']['props'] = dict()
_user['userPassword']['props']['minlength'] = 6
_user['userPassword']['props']['ascii'] = True
_user['userPassword']['required'] = True
_user['userPassword']['protected'] = False


_user['cn'] = dict()
_user['cn']['chain'] = 'field:label:error:text'
_user['cn']['required'] = True
_user['cn']['protected'] = False


_user['sn'] = dict()
_user['sn']['chain'] = 'field:label:error:text'
_user['sn']['required'] = True
_user['sn']['protected'] = False


_group['default'] = _user['default']


_group['id'] = _user['id']



from pyramid.view import view_config
from pyramid.i18n import (
    TranslationStringFactory,
    get_localizer,
)
from ..model.user import User
from ..model.group import Group

_ = TranslationStringFactory('cone.ugm')


###############################################################################
# local manager manage membership validation
###############################################################################

LM_TARGET_GID_NOT_ALLOWED = 0
LM_TARGET_UID_NOT_ALLOWED = 1
LM_TARGET_GID_IS_DEFAULT = 2


class ManageMembershipError(Exception):

    def __init__(self, reason, data):
        self.reason = reason
        self.data = data


def validate_add_users_to_groups(model, user_ids, group_ids):
    if not model.local_manager_consider_for_user:
        return
    lm_gids = model.local_manager_target_gids
    for group_id in group_ids:
        if not group_id in lm_gids:
            raise ManageMembershipError(LM_TARGET_GID_NOT_ALLOWED, group_id)
    lm_uids = model.local_manager_target_uids
    for user_id in user_ids:
        if not user_id in lm_uids:
            raise ManageMembershipError(LM_TARGET_UID_NOT_ALLOWED, user_id)


def validate_remove_users_from_groups(model, user_ids, group_ids):
    if not model.local_manager_consider_for_user:
        return
    lm_gids = model.local_manager_target_gids
    for group_id in group_ids:
        if not group_id in lm_gids:
            raise ManageMembershipError(LM_TARGET_GID_NOT_ALLOWED, group_id)
    lm_uids = model.local_manager_target_uids
    for user_id in user_ids:
        if not user_id in lm_uids:
            raise ManageMembershipError(LM_TARGET_UID_NOT_ALLOWED, user_id)
    adm_gid = model.local_manager_gid
    for group_id in group_ids:
        if model.local_manager_is_default(adm_gid, group_id):
            raise ManageMembershipError(LM_TARGET_GID_IS_DEFAULT, group_id)


###############################################################################
# Actions for User application node
###############################################################################

@view_config(name='delete_item', accept='application/json',
             renderer='json', context=User, permission='delete_user')
def delete_user_action(model, request):
    """Delete user from database.
    """
    try:
        users = model.parent.backend
        uid = model.model.name
        del users[uid]
        users()
        model.parent.invalidate()
        localizer = get_localizer(request)
        message = localizer.translate(
            _('delete_user_from_database',
              default="Deleted user '${uid}' from database.",
              mapping={'uid': uid}))
        return {'success': True,
                'message': message}
    except Exception, e:
        return {'success': False,
                'message': str(e)}


@view_config(name='add_item', accept='application/json',
             renderer='json', context=User, permission='manage_membership')
def user_add_to_group_action(model, request):
    """Add user to group.
    """
    group_id = request.params.get('id')
    if not group_id:
        group_ids = request.params.getall('id[]')
    else:
        group_ids = [group_id]
    try:
        user = model.model
        validate_add_users_to_groups(model, [user.id], group_ids)
        groups = user.root.groups
        for group_id in group_ids:
            groups[group_id].add(user.name)
        groups()
        model.parent.invalidate(user.name)
        localizer = get_localizer(request)
        message = localizer.translate(
            _('added_user_to_group',
              default="Added user '${uid}' to group '${gid}'.",
              mapping={'uid': user.id,
                  'gid': ', '.join(group_ids)}))
        return {'success': True,
                'message': message}
    except ManageMembershipError, e:
        if e.reason is not LM_TARGET_GID_NOT_ALLOWED:
            raise Exception(u"Unknown ManageMembershipError reason.")
        localizer = get_localizer(request)
        message = localizer.translate(
            _('lm_add_target_gid_not_allowed',
              default="Failed adding user '${uid}' to group '${gid}'. "
                      "Manage membership denied for target group.",
              mapping={'uid': user.id,
                       'gid': e.data}))
        return {'success': False,
                'message': message}
    except Exception, e:
        return {'success': False,
                'message': str(e)}


@view_config(name='remove_item', accept='application/json',
             renderer='json', context=User, permission='manage_membership')
def user_remove_from_group_action(model, request):
    """Remove user from group.
    """
    group_id = request.params.get('id')
    if not group_id:
        group_ids = request.params.getall('id[]')
    else:
        group_ids = [group_id]
    try:
        user = model.model
        validate_remove_users_from_groups(model, [user.id], group_ids)
        groups = user.root.groups
        for group_id in group_ids:
            del groups[group_id][user.name]
        groups()
        model.parent.invalidate(user.name)
        localizer = get_localizer(request)
        message = localizer.translate(
            _('removed_user_from_group',
              default="Removed user '${uid}' from group '${gid}'.",
              mapping={'uid': user.id,
                       'gid': ', '.join(group_ids)}))
        return {'success': True,
                'message': message}
    except ManageMembershipError, e:
        localizer = get_localizer(request)
        if e.reason is LM_TARGET_GID_NOT_ALLOWED:
            message = localizer.translate(
                _('lm_remove_target_gid_not_allowed',
                  default="Failed removing user '${uid}' from group '${gid}'. "
                          "Manage membership denied for target group.",
                  mapping={'uid': user.id,
                           'gid': e.data}))
        elif e.reason is LM_TARGET_GID_IS_DEFAULT:
            message = localizer.translate(
                _('lm_remove_target_gid_is_default',
                  default="Failed removing user '${uid}' from group '${gid}'. "
                          "Target group is default group of user.",
                  mapping={'uid': user.id,
                           'gid': e.data}))
        else:
            raise Exception(u"Unknown ManageMembershipError reason.")
        return {'success': False,
                'message': message}
    except Exception, e:
        return {'success': False,
                'message': str(e)}


###############################################################################
# Actions for Group application node
###############################################################################

@view_config(name='delete_item', accept='application/json',
             renderer='json', context=Group, permission='delete_group')
def delete_group_action(model, request):
    """Delete group from database.
    """
    try:
        groups = model.parent.backend
        uid = model.model.name
        del groups[uid]
        groups()
        model.parent.invalidate()
    except Exception, e:
        return {'success': False,
                'message': str(e)}
    localizer = get_localizer(request)
    message = localizer.translate(_('deleted_group',
                                    'Deleted group from database'))
    return {'success': True,
            'message': message}


@view_config(name='add_item', accept='application/json',
             renderer='json', context=Group, permission='manage_membership')
def group_add_user_action(model, request):
    """Add user to group.
    """
    user_id = request.params.get('id')
    if not user_id:
        user_ids = request.params.getall('id[]')
    else:
        user_ids = [user_id]
    try:
        group = model.model
        validate_add_users_to_groups(model, user_ids, [group.id])
        for user_id in user_ids:
            group.add(user_id)
        group()
        model.parent.invalidate(group.name)
        localizer = get_localizer(request)
        message = localizer.translate(
            _('added_user_to_group',
              default="Added user '${uid}' to group '${gid}'.",
              mapping={'uid': ', '.join(user_ids),
                  'gid': group.id}))
        return {'success': True,
                'message': message}
    except ManageMembershipError, e:
        if e.reason is not LM_TARGET_UID_NOT_ALLOWED:
            raise Exception(u"Unknown ManageMembershipError reason.")
        localizer = get_localizer(request)
        message = localizer.translate(
            _('lm_add_target_uid_not_allowed',
              default="Failed adding user '${uid}' to group '${gid}'. "
                      "Manage membership denied for user.",
              mapping={'uid': e.data,
                       'gid': group.id}))
        return {'success': False,
                'message': message}
    except Exception, e:
        return {'success': False,
                'message': str(e)}


@view_config(name='remove_item', accept='application/json',
             renderer='json', context=Group, permission='manage_membership')
def group_remove_user_action(model, request):
    """Remove user from group.
    """
    user_id = request.params.get('id')
    if not user_id:
        user_ids = request.params.getall('id[]')
    else:
        user_ids = [user_id]
    try:
        group = model.model
        validate_remove_users_from_groups(model, user_ids, [group.id])
        for user_id in user_ids:
            del group[user_id]
        group()
        model.parent.invalidate(group.name)
        localizer = get_localizer(request)
        message = localizer.translate(
            _('removed_user_from_group',
              default="Removed user '${uid}' from group '${gid}'.",
              mapping={'uid': ', '.join(user_ids),
                       'gid': group.id}))
        return {'success': True,
                'message': message}
    except ManageMembershipError, e:
        localizer = get_localizer(request)
        if e.reason is LM_TARGET_UID_NOT_ALLOWED:
            message = localizer.translate(
                _('lm_remove_target_uid_not_allowed',
                  default="Failed removing user '${uid}' from group '${gid}'. "
                          "Manage membership denied for user.",
                  mapping={'uid': e.data,
                           'gid': group.id}))
        elif e.reason is LM_TARGET_GID_IS_DEFAULT:
            message = localizer.translate(
                _('lm_remove_target_gid_is_default',
                  default="Failed removing user '${uid}' from group '${gid}'. "
                          "Target group is default group of user.",
                  mapping={'uid': ', '.join(user_ids),
                           'gid': e.data}))
        else:
            raise Exception(u"Unknown ManageMembershipError reason.")
        return {'success': False,
                'message': message}
    except Exception, e:
        return {'success': False,
                'message': str(e)}



import os
import ldap
import cone.ugm
from pyramid.i18n import TranslationStringFactory
from node.utils import instance_property
from node.ext.ldap import (
    LDAPProps,
    LDAPNode,
    testLDAPConnectivity,
)
from node.ext.ldap.ugm import (
    UsersConfig as LDAPUsersConfig,
    GroupsConfig as LDAPGroupsConfig,
    RolesConfig as LDAPRolesConfig,
)
from node.ext.ldap.ugm._api import EXPIRATION_DAYS
from cone.app.model import (
    BaseNode,
    XMLProperties,
    Metadata,
)
from .localmanager import LocalManagerConfigAttributes
from .utils import (
    ldap_cfg_file,
    localmanager_cfg_file,
)

_ = TranslationStringFactory('cone.ugm')


def _read_ugm_config():
    cfg_file = ldap_cfg_file()
    if not os.path.isfile(cfg_file):
        raise ValueError('Configuration file %s does not exist.' % cfg_file)
    ugm_config = XMLProperties(cfg_file)
    return ugm_config


def _invalidate_ugm_settings(model):
    settings = model.root['settings']
    settings['ugm_general']._xml_config = None
    settings['ugm_server']._ldap_props = None
    settings['ugm_server']._xml_config = None
    settings['ugm_users']._ldap_ucfg = None
    settings['ugm_users']._xml_config = None
    settings['ugm_groups']._ldap_gcfg = None
    settings['ugm_groups']._xml_config = None
    settings['ugm_roles']._ldap_gcfg = None
    settings['ugm_roles']._xml_config = None
    import cone.app
    cone.app.cfg.auth = None


class UgmSettings(BaseNode):

    def __call__(self):
        self.attrs()

    def invalidate(self):
        _invalidate_ugm_settings(self)
        self.root['users'].invalidate()
        self.root['groups'].invalidate()

    @property
    def attrs(self):
        return self._config

    @property
    def _config(self):
        if not hasattr(self, '_xml_config') or self._xml_config is None:
            self._xml_config = _read_ugm_config()
        return self._xml_config


class GeneralSettings(UgmSettings):

    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _('ugm_settings_node', 'UGM Settings')
        metadata.description = _('ugm_settings_node_description',
                                 'General user and group management settings')
        return metadata


class ServerSettings(UgmSettings):

    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _('ldap_props_node', 'LDAP Props')
        metadata.description = _('ldap_props_node_description',
                                 'LDAP properties')
        return metadata

    @property
    def ldap_connectivity(self):
        try:
            props = self.ldap_props
        except ValueError:
            return False
        return testLDAPConnectivity(props=props) == 'success'

    @property
    def ldap_props(self):
        if not hasattr(self, '_ldap_props') or self._ldap_props is None:
            config = self._config
            self._ldap_props = LDAPProps(
                uri=config.uri,
                user=config.user,
                password=config.password,
                cache=int(config.cache))
        return self._ldap_props


class UsersSettings(UgmSettings):

    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _('user_settings_node', 'Users Settings')
        metadata.description = _('user_settings_node_description',
                                 'LDAP users settings')
        return metadata

    @property
    def ldap_users_container_valid(self):
        try:
            node = LDAPNode(self._config.users_dn,
                            self.parent['ugm_server'].ldap_props)
            return len(node) >= 0
        except ldap.LDAPError:
            return False

    @property
    def ldap_ucfg(self):
        if not hasattr(self, '_ldap_ucfg') or self._ldap_ucfg is None:
            config = self._config
            map = dict()
            for key in config.users_aliases_attrmap.keys():
                map[key] = config.users_aliases_attrmap[key]
            for key in config.users_form_attrmap.keys():
                if key in ['id', 'login']:
                    continue
                map[key] = key
            if config.users_exposed_attributes:
                for key in config.users_exposed_attributes:
                    map[key] = key
            expiresAttr = None
            expiresUnit = EXPIRATION_DAYS
            # from general settings. be aware that all config options are
            # currently available on ``self._config``. This might change in
            # future and below settings must be read from general settings
            # then.
            if config.users_account_expiration == 'True':
                expiresAttr = config.users_expires_attr
                expiresUnit = int(config.users_expires_unit)
                map[expiresAttr] = expiresAttr
            if config.users_portrait == 'True':
                imageAttr = config.users_portrait_attr
                map[imageAttr] = imageAttr
            import cone.ugm.model
            self._ldap_ucfg = LDAPUsersConfig(
                baseDN=config.users_dn,
                attrmap=map,
                scope=int(config.users_scope),
                queryFilter=config.users_query,
                objectClasses=config.users_object_classes,
                defaults=cone.ugm.model.factory_defaults.user,
                expiresAttr=expiresAttr,
                expiresUnit=expiresUnit)
        return self._ldap_ucfg


class GroupsSettings(UgmSettings):

    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _('group_settings_node', 'Groups Settings')
        metadata.description = _('group_settings_node_description',
                                 'LDAP groups settings')
        return metadata

    @property
    def ldap_groups_container_valid(self):
        try:
            node = LDAPNode(self._config.groups_dn,
                            self.parent['ugm_server'].ldap_props)
            return len(node) >= 0
        except ldap.LDAPError:
            return False

    @property
    def ldap_gcfg(self):
        if not hasattr(self, '_ldap_gcfg') or self._ldap_gcfg is None:
            config = self._config
            map = dict()
            for key in config.groups_aliases_attrmap.keys():
                map[key] = config.groups_aliases_attrmap[key]
            for key in config.groups_form_attrmap.keys():
                if key in ['id']:
                    continue
                map[key] = key
            import cone.ugm.model
            self._ldap_gcfg = LDAPGroupsConfig(
                baseDN=config.groups_dn,
                attrmap=map,
                scope=int(config.groups_scope),
                queryFilter=config.groups_query,
                objectClasses=config.groups_object_classes,
                #member_relation=config.groups_relation,
                defaults=cone.ugm.model.factory_defaults.group,
                )
        return self._ldap_gcfg


class RolesSettings(UgmSettings):

    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _('role_settings_node', 'Roles Settings')
        metadata.description = _('role_settings_node_description',
                                 'LDAP roles settings')
        return metadata

    @property
    def ldap_roles_container_valid(self):
        try:
            node = LDAPNode(self._config.roles_dn,
                            self.parent['ugm_server'].ldap_props)
            return len(node) >= 0
        except ldap.LDAPError:
            return False

    @property
    def ldap_rcfg(self):
        if not hasattr(self, '_ldap_rcfg') or self._ldap_rcfg is None:
            config = self._config
            map = dict()
            for key in config.roles_aliases_attrmap.keys():
                map[key] = config.roles_aliases_attrmap[key]
            for key in config.roles_form_attrmap.keys():
                if key in ['id']:
                    continue
                map[key] = key
            import cone.ugm.model
            self._ldap_rcfg = LDAPRolesConfig(
                baseDN=config.roles_dn,
                attrmap=map,
                scope=int(config.roles_scope),
                queryFilter=config.roles_query,
                objectClasses=config.roles_object_classes,
                #member_relation=config.roles_relation,
                defaults=cone.ugm.model.factory_defaults.role,
                )
        return self._ldap_rcfg


class LocalManagerSettings(BaseNode):

    def attributes_factory(self, name=None, parent=None):
        return LocalManagerConfigAttributes(localmanager_cfg_file())

    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _('localmanager_settings_node',
                           'Local Manager Settings')
        metadata.description = _('localmanager_settings_node_description',
                                 'Local Manager Settings')
        return metadata

    @property
    def enabled(self):
        general_settings = self.root['settings']['ugm_general']
        return general_settings.attrs.users_local_management_enabled == 'True'

    def __call__(self):
        self.attrs()



from pyramid.i18n import (
    TranslationStringFactory,
    get_localizer,
)
from yafowil.base import (
    factory,
    UNSET,
)
from cone.app.browser.utils import make_url

_ = TranslationStringFactory('cone.ugm')


class PrincipalForm(object):

    form_name = None

    @property
    def form_attrmap(self):
        raise NotImplementedError(u"Abstract principal form does not provide "
                                  u"``form_attrmap``")

    @property
    def form_field_definitions(self):
        raise NotImplementedError(u"Abstract principal form does not provide "
                                  u"``form_field_definitions``")

    def prepare(self):
        resource = self.action_resource

        # load props befor edit form is rendered.
        # XXX: this is LDAP world, not generic UGM
        if resource == 'edit':
            self.model.attrs.context.load()

        action = make_url(self.request, node=self.model, resource=resource)
        form = factory(
            u'form',
            name=self.form_name,
            props={
                'action': action,
            })
        attrmap = self.form_attrmap
        if not attrmap:
            return form
        schema = self.form_field_definitions
        default = schema['default']
        localizer = get_localizer(self.request)
        for key, val in attrmap.items():
            field = schema.get(key, default)
            chain = field.get('chain', default['chain'])
            props = dict()
            props['label'] = _(val, val)
            if field.get('required'):
                req = _('no_field_value_defined',
                        default='No ${field} defined',
                        mapping={'field': localizer.translate(_(val, val))})
                props['required'] = req
            props.update(field.get('props', dict()))
            value = UNSET
            mode = 'edit'
            if resource == 'edit':
                if field.get('protected'):
                    mode = 'display'
                value = self.model.attrs.get(key, u'')
            custom = field.get('custom', dict())
            custom_parsed = dict()
            if custom.keys():
                for k, v in custom.items():
                    val_parsed = list()
                    for c_chain in v:
                        chain_parsed = list()
                        for part in c_chain:
                            if isinstance(part, basestring):
                                if not part.startswith('context.'):
                                    raise Exception(
                                        u"chain callable definition invalid")
                                attrname = part[part.index('.') + 1:]
                                clb = getattr(self, attrname)
                            else:
                                clb = part
                            chain_parsed.append(clb)
                        val_parsed.append(chain_parsed)
                    custom_parsed[k] = tuple(val_parsed)
            form[key] = factory(
                chain,
                value=value,
                props=props,
                custom=custom_parsed,
                mode=mode)
        form['save'] = factory(
            'submit',
            props={
                'action': 'save',
                'expression': True,
                'handler': self.save,
                'next': self.next,
                'label': _('save', 'Save'),
            })
        if resource == 'add':
            form['cancel'] = factory(
                'submit',
                props={
                    'action': 'cancel',
                    'expression': True,
                    'handler': None,
                    'next': self.next,
                    'label': _('cancel', 'Cancel'),
                    'skip': True,
                })
        self.form = form



import logging
from pyramid.security import has_permission
from pyramid.i18n import TranslationStringFactory
from cone.tile import (
    tile,
    Tile,
)
from cone.app.browser.utils import (
    make_url,
    make_query,
)
from ..model.users import Users
from .listing import PrincipalsListing

logger = logging.getLogger('cone.ugm')
_ = TranslationStringFactory('cone.ugm')


@tile('leftcolumn', 'templates/left_column.pt',
      interface=Users, permission='view')
class UsersLeftColumn(Tile):

    add_label = _('add_user', 'Add User')

    @property
    def add_target(self):
        return make_url(self.request,
                        node=self.model.root['users'],
                        query=make_query(factory=u'user'))

    @property
    def can_add(self):
        return has_permission('add_user', self.model, self.request)


@tile('rightcolumn', interface=Users, permission='view')
class UsersRightColumn(Tile):

    def render(self):
        return u'<div class="column right_column">&nbsp;</div>'


@tile('columnlisting', 'templates/column_listing.pt',
      interface=Users, permission='view')
class UsersColumnListing(PrincipalsListing):

    slot = 'leftlisting'
    list_columns = PrincipalsListing.user_list_columns
    listing_attrs = PrincipalsListing.user_attrs
    listing_criteria = PrincipalsListing.user_listing_criteria
    sort_attr = PrincipalsListing.user_default_sort_column
    css = 'users'
    batchname = 'leftbatch'
    delete_label = _('delete_user', 'Delete User')
    delete_permission = 'delete_user'

    @property
    def current_id(self):
        return getattr(self.request, '_curr_listing_id', None)



from cone.app.browser.batch import Batch
from cone.app.browser.utils import (
    nodepath,
    make_query,
    make_url,
)


class ColumnBatch(Batch):
    """Abstract UGM column batch.
    """

    def __init__(self, name, items, slicesize):
        self.name = name
        #self.name = 'columnbatch'
        self.path = None
        self.attribute = 'render'
        self.items = items
        self.slicesize = slicesize

    @property
    def display(self):
        return len(self.vocab) > 1

    @property
    def vocab(self):
        ret = list()
        path = nodepath(self.model)
        count = len(self.items)
        pages = count / self.slicesize
        if count % self.slicesize != 0:
            pages += 1
        current = self.request.params.get('b_page', '0')
        for i in range(pages):
            query = make_query(b_page=str(i))
            url = make_url(self.request, path=path, query=query)
            ret.append({
                'page': '%i' % (i + 1),
                'current': current == str(i),
                'visible': True,
                'url': url,
            })
        return ret

    @property
    def firstpage(self):
        return None

    @property
    def prevpage(self):
        return None

    @property
    def nextpage(self):
        return None

    @property
    def lastpage(self):
        return None

    @property
    def leftellipsis(self):
        return ''

    @property
    def rightellipsis(self):
        return ''



import os
from setuptools import (
    setup,
    find_packages,
)


version = '0.9.6'
shortdesc = 'cone.ugm'
longdesc = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()


setup(name='cone.ugm',
      version=version,
      description=shortdesc,
      long_description=longdesc,
      classifiers=[
          'Environment :: Web Environment',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
      ],
      keywords='',
      author='BlueDynamics Alliance',
      author_email='dev@bluedynamics.com',
      url=u'http://github.com/bluedynamics/cone.ugm',
      license='GNU General Public Licence',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['cone'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'cone.app',
          'yafowil.widget.array',
          'yafowil.widget.autocomplete',
          'yafowil.widget.datetime',
          'yafowil.widget.dict',
          'yafowil.widget.image',
          'node.ext.ldap',
          #'repoze.profile',
      ],
      extras_require = dict(
          test=[
                'interlude',
          ]
      ),
      tests_require=[
          'interlude',
      ],
      test_suite = "cone.ugm.tests.test_suite",
      message_extractors = {
          '.': [
              ('**.py', 'lingua_python', None),
              ('**.pt', 'lingua_xml', None),
          ]
      }
      )



import os
import logging
import cone.app
from pyramid.security import (
    Everyone,
    Allow,
    Deny,
    ALL_PERMISSIONS,
)
from cone.app.security import acl_registry
from cone.app.model import Properties
from .model.user import User
from .model.users import Users
from .model.group import Group
from .model.groups import Groups
from .model.settings import (
    GeneralSettings,
    ServerSettings,
    UsersSettings,
    GroupsSettings,
    RolesSettings,
    LocalManagerSettings,
)
from .model.users import users_factory
from .model.groups import groups_factory
from .browser import static_resources
from node.ext.ldap.ugm import Ugm

logger = logging.getLogger('cone.ugm')

# custom UGM styles
cone.app.cfg.merged.css.protected.append((static_resources, 'styles.css'))

# custom UGM javascript
cone.app.cfg.merged.js.protected.append((static_resources,
                                         'jQuery.sortElements.js'))
cone.app.cfg.merged.js.protected.append((static_resources, 'naturalSort.js'))
cone.app.cfg.merged.js.protected.append((static_resources, 'ugm.js'))

# layout configuration
cone.app.cfg.layout.livesearch = False
cone.app.cfg.layout.pathbar = False
cone.app.cfg.layout.sidebar_left = []

# UGM settings
cone.app.register_plugin_config('ugm_general', GeneralSettings)
cone.app.register_plugin_config('ugm_server', ServerSettings)
cone.app.register_plugin_config('ugm_users', UsersSettings)
cone.app.register_plugin_config('ugm_groups', GroupsSettings)
cone.app.register_plugin_config('ugm_roles', RolesSettings)
cone.app.register_plugin_config('ugm_localmanager', LocalManagerSettings)

# Users container
cone.app.register_plugin('users', users_factory)

# Groups container
cone.app.register_plugin('groups', groups_factory)

# security
management_permissions = [
    'add', 'edit', 'delete',
]
user_management_permissions = [
    'add_user', 'edit_user', 'delete_user', 'manage_expiration',
]
group_management_permissions = [
    'add_group', 'edit_group', 'delete_group',
]
admin_permissions = [
    'view', 'manage_membership', 'view_portrait',
] + management_permissions \
  + user_management_permissions \
  + group_management_permissions
ugm_default_acl = [
    (Allow, 'role:editor', ['view', 'manage_membership']),
    (Allow, 'role:admin', admin_permissions),
    (Allow, 'role:manager', admin_permissions + ['manage']),
    (Allow, Everyone, ['login']),
    (Deny, Everyone, ALL_PERMISSIONS),
]
ugm_user_acl = [
    (Allow, 'system.Authenticated', ['view_portrait']),
] + ugm_default_acl

# register default acl's
# XXX: define permissions referring users, user, groups respective group only,
acl_registry.register(ugm_user_acl, User, 'user')
acl_registry.register(ugm_default_acl, Users, 'users')
acl_registry.register(ugm_default_acl, Group, 'group')
acl_registry.register(ugm_default_acl, Groups, 'groups')


# application startup hooks
def initialize_ugm(config, global_config, local_config):
    """Initialize UGM.
    """
    # add translation
    config.add_translation_dirs('cone.ugm:locale/')

    # static resources
    config.add_view('cone.ugm.browser.static_resources',
                    name='cone.ugm.static')

cone.app.register_main_hook(initialize_ugm)


def initialize_auth_impl(config, global_config, local_config):
    """Initialize LDAP based UGM implementation for cone.app as
    authentication implementation.

    XXX: move to cone.ldap later
    """
    os.environ['LDAP_CFG_FILE'] = local_config.get('cone.ugm.ldap_config', '')
    os.environ['LOCAL_MANAGER_CFG_FILE'] = \
        local_config.get('cone.ugm.localmanager_config', '')
    import cone.ugm
    ldap_auth = local_config.get('cone.auth_impl') == 'node.ext.ldap'
    if not ldap_auth:
        return
    reset_ldap_auth_impl()

cone.app.register_main_hook(initialize_auth_impl)


def reset_auth_impl():
    """LDAP only ATM.
    """
    return reset_ldap_auth_impl()


def reset_ldap_auth_impl():
    """XXX: Move to ``cone.ldap``.
    """
    import cone.app
    root = cone.app.get_root()
    settings = root['settings']
    server_settings = settings['ugm_server']
    if not server_settings.ldap_connectivity:
        logger.error(u"Could not initialize authentication implementation. "
                     u"LDAP Server is not available or invalid credentials.")
        return
    props = server_settings.ldap_props
    users_settings = settings['ugm_users']
    if not users_settings.ldap_users_container_valid:
        logger.error(u"Could not initialize authentication implementation. "
                     u"Configured users container invalid.")
        return
    ucfg = users_settings.ldap_ucfg
    groups_settings = settings['ugm_groups']
    gcfg = None
    if groups_settings.ldap_groups_container_valid:
        gcfg = groups_settings.ldap_gcfg
    else:
        logger.warning(u"Configured groups container invalid.")
    roles_settings = settings['ugm_roles']
    rcfg = None
    if roles_settings.ldap_roles_container_valid:
        rcfg = roles_settings.ldap_rcfg
    else:
        logger.warning(u"Configured roles container invalid.")
    ugm = Ugm(name='ldap_ugm', props=props, ucfg=ucfg, gcfg=gcfg, rcfg=rcfg)
    cone.app.cfg.auth = ugm
    return ugm
